const Time = require("../functions/Time");
const monitoringController = require("../controller/MonitoringController");

class MonitoringSocket {
    clients = [];
    twentySeconds = Time.getMilliSeconds(20, "seconds");
    twentyMinutes = Time.getMilliSeconds(20, "minutes");
    intervals = [];
    clientIndicatorsId = [];
    monitoringResult = {};

    async removeClientBySocketId(socketId) {
        let clients = await this.getClients();
        let idxToRemove = clients.findIndex((cl) => cl.id === socketId);
        if (idxToRemove > -1) {
            this.clients.splice(idxToRemove, 1);
            return true;
        }
        return false;
    }
    setSlaveList(slavelist) {
        this.slavelist = slavelist;
    }

    async getSocketUpdaterByUnit(CDFILIAL) {
        return this.getSocketIo()
            .in("ASSISTANT_CLIENT_ROOM")
            .fetchSockets()
            .then((allSockets) => {
                const updaterUnitSocket = allSockets.find((socket) => {
                    return socket.CDFILIAL == CDFILIAL;
                });
                return updaterUnitSocket;
            });
    }

    setSocketIo(io) {
        this.io = io;
    }

    getSocketIo() {
        return this.io;
    }

    getClientIndicatorsId() {
        return this.clientIndicatorsId;
    }

    updateStatusClientIndicators(unit) {
        this.getClientIndicatorsId().forEach((id) => {
            unit[id] = "N";
        });
    }

    getRoom(roomName) {
        return this.getSocketIo().to(roomName);
    }

    addClient(client) {
        this.clients.push(client);
    }

    async getClients() {
        // this.io.in('ASSISTANT_CLIENT_ROOM').fetchSockets().then(resp => console.log(resp))
        return this.clients; // temp solution
    }

    createIntervalValidateInactiveUser(socketId, socketIo) {
        let interval = setInterval(async () => {
            let clients = await this.getClients();
            clients.forEach((portais) => {
                let inactiveUser =
                    portais.time + this.twentyMinutes < new Date().getTime();
                if (inactiveUser) {
                    /** Client não atualizou a conexão, remove conexao */
                    socketIo.sockets.sockets.forEach((socket) => {
                        if (socket.id === portais.id) {
                            console.log("Desconectei o: ", socket.id);
                            socket.disconnect(true);
                        }
                    });
                }
            });
        }, this.twentySeconds);
        this.intervals.push({
            id: socketId,
            interval,
        });
    }

    removeIntervalValidateInactiveUser(socketId) {
        this.intervals.some((interval, index) => {
            if (interval.id === socketId) {
                clearInterval(interval.interval);
                this.intervals.splice(index, 1);
                return true;
            }
            return false;
        });
    }

    registerEvents(socket) {
        socket.on("registerIndicatorsAssistantClient", (clientIndicatorsId) => {
            this.clientIndicatorsId = clientIndicatorsId;
        });

        socket.on("detailMonitoring", (config) => {
            const detail = this.monitoringResult[config.CDFILIAL]
                ? this.monitoringResult[config.CDFILIAL][config.ID]
                : [];
            socket.emit("detailMonitoringResult", detail);
        });

        //   socket.removeAllListeners('execMonitoring');
        socket.on("execMonitoring", async (config) => {
            // Socket das lojas conectadas
            let clients = await this.getClients();
            clients.forEach((item) => {
                let newConfig = { ...config, CDFILIAL: item.socket.CDFILIAL };
                // A => Execução na base do Assistant
                // L => Execução na base Lojas, envia a requisição e aguarda o retorno para emitir para o Assistant
                // I => Execução de rotinas criadas internamente que não podem ser alteradas via sistema (SAT, por exemplo)
                // E => Execução de requisição HTTP nas Lojas, envia a requisição e aguarda o retorno para emitir para o Assistant
                // S => Verificação se Serviços do Windows na Loja estão em execução
                const functionPerType = {
                    A: this.monitoringDataBaseCloud,
                    L: this.monitoringDataBaseLoja,
                    I: this.monitoringAssistantInterno,
                    E: this.monitoringApacheLoja,
                    S: this.monitoringServicesLoja,
                };
                const functionToCall =
                    functionPerType[config.TIPO] || (() => {});
                functionToCall.apply(this, [newConfig, item.socket]);
            });
        });

        socket.on(`MONITORING_RESULT_APACHE`, (result) => {
            const { ID, NOME, CDFILIAL } = result.data;
            const config = { ID, NOME, CDFILIAL };
            if (!result.isError) {
                const isError = result.response.rows.some((element) => {
                    return element.STATUS == "N";
                });
                if (!isError) {
                    result.response.rows = [];
                }
                console.log(
                    `Filial: ${CDFILIAL} | HTTP Loja SUC: ${ID} - ${NOME}`
                );
                this.setMonitoringResult(config, result.response, isError);
            } else {
                const response = {
                    metaData: [{ name: "ERRO" }],
                    rows: [[result.response]],
                };
                console.log(
                    `Filial:', ${CDFILIAL} | HTTP Loja ERR: ${ID} - ${NOME} | Message: ${result.response}`
                );
                this.setMonitoringResult(config, response, true);
            }
        });

        socket.on(`MONITORING_RESULT_LOCAL_DB`, (result) => {
            const { ID, NOME, CDFILIAL } = result.data;
            const config = { ID, NOME, CDFILIAL };
            if (!result.isError) {
                console.log(
                    `Filial: ${CDFILIAL} | SQL Loja SUC: ${ID} - ${NOME} | Quantidade: ${result.response.rows.length}`
                );
                this.setMonitoringResult(config, result.response, false);
            } else {
                const response = {
                    metaData: [{ name: "ERRO" }],
                    rows: [[result.response]],
                };
                console.log(
                    `Filial: ${CDFILIAL} | SQL Loja ERR: ${ID} - ${NOME} | Message: ${result.response}`
                );
                console.log(
                    "Filial:",
                    CDFILIAL,
                    "| SQL Loja ERR:",
                    ID + " - " + NOME,
                    "| Message:",
                    result.response
                );
                this.setMonitoringResult(config, response, true);
            }
        });

        socket.on(`MONITORING_RESULT_LOCAL_SERVICES`, (result) => {
            const { ID, NOME, CDFILIAL } = result.data;
            const config = { ID, NOME, CDFILIAL };
            if (!result.isError) {
                const isError = result.response.rows.some((element) => {
                    return element.STATUS == "N";
                });
                if (!isError) {
                    result.response.rows = [];
                }
                console.log(
                    "Filial:",
                    CDFILIAL,
                    "| Services Loja SUC:",
                    ID + " - " + NOME
                );
                this.setMonitoringResult(config, result.response, isError);
            } else {
                const response = {
                    metaData: [{ name: "ERRO" }],
                    rows: [[result.response]],
                };
                console.log(
                    "Filial:",
                    CDFILIAL,
                    "| Services Loja ERR:",
                    ID + " - " + NOME,
                    "| Message:",
                    result.response
                );
                this.setMonitoringResult(config, response, true);
            }
        });

        socket.on("MONITORING_RESULT_SAT", (result) => {
            const { ID, NOME, CDFILIAL } = result.data;
            const config = { ID, NOME, CDFILIAL };
            const curUnit = this.getUnitInSlavelist(CDFILIAL);
            if (!result.isError) {
                const isSatError = result.response.rows.some((element) => {
                    return !element.sat;
                });
                if (!isSatError) {
                    result.response.rows = [];
                }
                console.log(
                    `Filial: ${CDFILIAL} | Exec SAT SUC: ${ID} - ${NOME} | Error: ${isSatError}`
                );
                this.setMonitoringResult(config, result.response, isSatError);
                if (curUnit) {
                    curUnit.sat = isSatError ? "E" : "S";
                }
            } else {
                const response = {
                    metaData: [{ name: "ERRO" }],
                    rows: [[result.response]],
                };
                console.log(
                    `Filial: ${CDFILIAL} | Exec SAT ERR: ${ID} - ${NOME} | Message: `,
                    result.response
                );
                this.setMonitoringResult(config, response, true);
                if (curUnit) {
                    curUnit.sat = "E";
                }
            }
        });
    }
    monitoringDataBaseCloud(config) {
        const { ID, NOME, CDFILIAL } = config;
        monitoringController
            .execCommand(config.COMANDO, { CDFILIAL })
            .then((response) => {
                console.log(
                    `Filial: ${CDFILIAL} | DB Cloud SUC: ${ID} - ${NOME} | Quantidade: ${response.rows.length}`
                );
                this.setMonitoringResult(config, response, false);
            })
            .catch((error) => {
                const response = {
                    metaData: [{ name: "ERRO" }],
                    rows: [[error.message]],
                };
                console.log(
                    `Filial: ${CDFILIAL} | DB Cloud ERR: ${ID} - ${NOME} | Message: ${error.message}`
                );
                this.setMonitoringResult(config, response, true);
            });
    }
    getUnitInSlavelist(CDFILIAL) {
        return this.slavelist.find((unit) => unit.id === CDFILIAL);
    }
    setMonitoringResult(config, response, isError) {
        //Atualiza a lista a ser usada no grid de monitoramento.
        const { CDFILIAL, ID } = config;
        let curUnit = this.getUnitInSlavelist(CDFILIAL);
        if (curUnit) {
            curUnit[ID] = isError || response.rows.length > 0 ? "N" : "S";
        }
        //Atualiza os dados do modal de detalhes de cada monitoramento.
        this.monitoringResult[CDFILIAL] = this.monitoringResult[CDFILIAL] || {};
        this.monitoringResult[CDFILIAL][ID] = response;

        //Envia informações atualizadas para a tela de monitoramento.
        this.getRoom("ASSISTANT_MASTER_ROOM").emit(ID, this.slavelist);
    }

    monitoringDataBaseLoja(config, socketClient) {
        //Envia para a Loja
        socketClient.emit("execMonitoringSQL", config, socketClient.rooms);

        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        // socketClient.removeAllListeners(`${config.ID}_result`);
    }

    monitoringApacheLoja(config, socketClient) {
        //Envia para a Loja
        socketClient.emit("execMonitoringHttp", config, socketClient.rooms);
        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        // socketClient.removeAllListeners(`${config.ID}_result`);
    }

    monitoringServicesLoja(config, socketClient) {
        //Envia para a Loja
        socketClient.emit("execMonitoringServices", config, socketClient.rooms);

        //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
    }

    monitoringAssistantInterno(config, socketClient) {
        const { CDFILIAL } = config;
        const curUnit = this.getUnitInSlavelist(CDFILIAL);
        if (config.NOME == "SAT" && curUnit.sat !== "N") {
            //Envia para a Loja
            socketClient.emit("satMonitoring", config, socketClient.rooms);

            //Recebe a resposta da Loja, sempre atualizando o listener para não duplicar
        }
    }
}

module.exports = new MonitoringSocket();